/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import ch.qos.logback.classic.boolex.JaninoEventEvaluator;
import ch.qos.logback.core.filter.EvaluatorFilter;
import ch.qos.logback.core.spi.FilterReply;
import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.model.EventFilter;
import ch.qos.logback.eclipse.views.FilterContentProvider;

/**
 * @author S&eacute;bastien Pennec
 */
public class LogbackFilterDialog extends Dialog {

  private TableViewer viewer;
  private TableColumn codeColumn;
  private TableColumn onMatchColumn;
  private TableColumn onMismatchColumn;
  
  private Text expressionField;
  private Text matchField;
  private Text mismatchField;

  private EvaluatorFilter current = null;
  
  public LogbackFilterDialog(Shell parentShell) {
    super(parentShell);
    
    setShellStyle(SWT.DIALOG_TRIM | SWT.RESIZE | SWT.APPLICATION_MODAL);
  }

  protected Control createDialogArea(Composite parent) {
    Composite container = (Composite) super.createDialogArea(parent);
    container.setSize(900, 3000);
    
    FillLayout layout = new FillLayout();
    layout.type = SWT.VERTICAL;
    container.setLayout(layout);

    createTable(container);
    Composite editComposite = new Composite(container, SWT.BORDER);
    editComposite.setSize(900, 1000);
    createEditionUIPart(editComposite);
    createButtons(editComposite);
    
    return container;
  }
  
  private void createEditionUIPart(Composite container) {
    final GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 3;
    container.setLayout(gridLayout);
    
    final Label infoLabel = new Label(container, SWT.NONE);
    infoLabel.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 2, 1));
    infoLabel.setText(Messages.FilterDialogInfoLabel);
    final Button moveUp = new Button(container, SWT.PUSH);
    moveUp.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false, 1, 1));
    moveUp.setText(Messages.Up);
    moveUp.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        super.widgetSelected(e);
        EventFilter.moveFilterUp(current);
        viewer.refresh();
      }
    });

    final Label infoLabel2 = new Label(container, SWT.NONE);
    infoLabel2.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 2, 1));
    infoLabel2.setText(Messages.FilterDialogInfoLabel2);
    final Button moveDown = new Button(container, SWT.PUSH);
    moveDown.setLayoutData(new GridData(GridData.END, GridData.CENTER, false, false, 1, 1));
    moveDown.setText(Messages.Down);
    moveDown.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        super.widgetSelected(e);
        EventFilter.moveFilterDown(current);
        viewer.refresh();
      }
    });
    
    expressionField = new Text(container, SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
    GridData gridData = new GridData();
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.heightHint = 75;
    gridData.widthHint = 320;
    gridData.horizontalSpan = 3;
    gridData.verticalSpan = 2;
    expressionField.setLayoutData(gridData);
    
    Composite cmp = new Composite(container, SWT.NONE);
    cmp.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false, 3, 1));
    GridLayout grd = new GridLayout();
    grd.numColumns = 2;
    cmp.setLayout(grd);
    final Label infoLabel3 = new Label(cmp, SWT.NONE);
    infoLabel3.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 1, 1));
    infoLabel3.setText(Messages.FilterDialogActionOnFilterMatchLabel);
    matchField = new Text(cmp, SWT.BORDER);
//    Rectangle bounds2 = matchField.getBounds();
//    bounds2.width = 150;
    matchField.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));

    final Label infoLabel4 = new Label(cmp, SWT.NONE);
    infoLabel4.setLayoutData(new GridData(GridData.BEGINNING, GridData.CENTER, false, false, 1, 1));
    infoLabel4.setText(Messages.FilterDialogActionOnFilterMismatchLabel);
    mismatchField = new Text(cmp, SWT.BORDER);
//    Rectangle bounds3 = mismatchField.getBounds();
//    bounds3.width = 150;
    mismatchField.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, true, false));
  }
  
  private void createButtons(Composite container) {

    final Button saveButton = new Button(container, SWT.PUSH);
    saveButton.setText(Messages.Save);
    saveButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        super.widgetSelected(e);
        saveCurrentFilter();
        viewer.refresh();
      }
    });
    
    final Button newButton = new Button(container, SWT.PUSH);
    newButton.setText(Messages.New);
    newButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        super.widgetSelected(e);
        viewer.setSelection(StructuredSelection.EMPTY);
        createNewFilter();
        viewer.refresh();
      }
    });
    
    final Button deleteButton = new Button(container, SWT.PUSH);
    deleteButton.setText(Messages.Delete);
    deleteButton.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        super.widgetSelected(e);
        deleteCurrentFilter();
        viewer.refresh();
      }
    });
  }

  private void createTable(Composite container) {
    viewer = new TableViewer(container, SWT.H_SCROLL | SWT.V_SCROLL | SWT.SINGLE
        | SWT.FULL_SELECTION);
    
    FilterContentProvider.getProvider().setViewer(viewer);
    
    final Table table = viewer.getTable();

    codeColumn = new TableColumn(table, SWT.LEFT);
    codeColumn.setText(Messages.FilterDialogExpression);
    codeColumn.setWidth(200);

    onMatchColumn = new TableColumn(table, SWT.LEFT);
    onMatchColumn.setText(Messages.FilterDialogOnMatch);
    onMatchColumn.setWidth(80);

    onMismatchColumn = new TableColumn(table, SWT.LEFT);
    onMismatchColumn.setText(Messages.FilterDialogOnMismatch);
    onMismatchColumn.setWidth(80);

    table.setHeaderVisible(true);
    table.setLinesVisible(true);
    
    viewer.setContentProvider(FilterContentProvider.getProvider());
    viewer.setLabelProvider(FilterContentProvider.getProvider());
    viewer.setInput(FilterContentProvider.getProvider());
    
    viewer.addSelectionChangedListener(new ISelectionChangedListener() {

      public void selectionChanged(SelectionChangedEvent event) {
        StructuredSelection sel = (StructuredSelection)event.getSelection();
        if (!sel.equals(StructuredSelection.EMPTY)) {
          editFilter(sel.getFirstElement());
        }
      }
    });
  }
  
  private void editFilter(Object element) {
    current = (EvaluatorFilter)element;
    JaninoEventEvaluator eval =  (JaninoEventEvaluator)current.getEvaluator();
    expressionField.setText(eval.getExpression());
    matchField.setText(current.getOnMatch().toString());
    mismatchField.setText(current.getOnMismatch().toString());
  }
  
  private void saveCurrentFilter() {
    current.stop();
    JaninoEventEvaluator eval = (JaninoEventEvaluator)current.getEvaluator();
    eval.stop();
    String oldExp = eval.getExpression();
    eval.setExpression(expressionField.getText());
    current.setOnMatch(FilterReply.valueOf(matchField.getText()));
    current.setOnMismatch(FilterReply.valueOf(mismatchField.getText()));
    eval.start();
    if (!eval.isStarted()) {
      eval.stop();
      eval.setExpression(oldExp);
      eval.start();
      MessageDialog.openInformation(getShell(), Messages.IncorrectExpressionErrorDialogTitle, 
              Messages.ExpressionIncorrectError);
      return;
    }
    current.start();
    EventFilter.add(current);
    emptyEditionUI();
  }

  private void emptyEditionUI() {
    current = null;
    expressionField.setText("");//$NON-NLS-1$
    expressionField.redraw();
    matchField.setText("");//$NON-NLS-1$
    matchField.redraw();
    mismatchField.setText("");//$NON-NLS-1$
    mismatchField.redraw();
  }
  
  private void createNewFilter() {
    current = FilterContentProvider.getProvider().createNewFilter();
    expressionField.setText(Messages.FilterDialogExpressionFieldDefaultMessage);
    matchField.setText(Messages.FilterDialogOnMatchFieldDefaultMessage);
    mismatchField.setText(Messages.FilterDialogOnMismatchFieldDefaultMessage);
  }

  private void deleteCurrentFilter() {
    current.stop();
    JaninoEventEvaluator eval = (JaninoEventEvaluator) current.getEvaluator();
    eval.stop();
    FilterContentProvider.getProvider().remove(current);
    emptyEditionUI();
  }

  protected void configureShell(Shell newShell) {
    super.configureShell(newShell);
    newShell.setText(Messages.FilterDialogTitle);
  }
}
