/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IMemento;

import ch.qos.logback.classic.boolex.JaninoEventEvaluator;
import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.core.Context;
import ch.qos.logback.core.ContextBase;
import ch.qos.logback.core.filter.EvaluatorFilter;
import ch.qos.logback.core.spi.FilterReply;
import ch.qos.logback.eclipse.Messages;

/**
 * @author S&eacute;bastien Pennec
 */
public class EventFilter {

  private static final String TAG_FILTER = "filter"; //$NON-NLS-1$
  private static final String TAG_EXPRESSION = "expression"; //$NON-NLS-1$
  private static final String TAG_ONMATCH = "onMatch"; //$NON-NLS-1$
  private static final String TAG_ONMISMATCH = "onMismatch"; //$NON-NLS-1$

  private static ContextBase context = new ContextBase();

  private static List<EvaluatorFilter> filterList = new ArrayList<EvaluatorFilter>();

  public static boolean filter(LoggingEvent event) {
    if (filterList.isEmpty()) {
      return true;
    }

    Iterator<EvaluatorFilter> it = filterList.iterator();
    EvaluatorFilter filter;
    while (it.hasNext()) {
      filter = it.next();
      FilterReply reply = filter.decide(event);
      if (reply == FilterReply.DENY) {
        return false;
      } else if (reply == FilterReply.ACCEPT) {
        return true;
      }
    }

    return true;
  }

  public static Context getContext() {
    return context;
  }

  public static List<EvaluatorFilter> getAllFilters() {
    return filterList;
  }

  public static void add(EvaluatorFilter filter) {
    if (!filterList.contains(filter)) {
      filterList.add(filter);
    }
  }

  public static void remove(EvaluatorFilter filter) {
    if (filterList.contains(filter)) {
      filterList.remove(filter);
    }
  }

  public static void saveState(IMemento memento) {
    Iterator<EvaluatorFilter> it = filterList.iterator();
    EvaluatorFilter filter;
    JaninoEventEvaluator eval;
    while (it.hasNext()) {
      filter = it.next();
      eval = (JaninoEventEvaluator) filter.getEvaluator();
      IMemento mem = memento.createChild(TAG_FILTER);
      mem.putString(TAG_EXPRESSION, eval.getExpression());
      mem.putString(TAG_ONMATCH, filter.getOnMatch().toString());
      mem.putString(TAG_ONMISMATCH, filter.getOnMismatch().toString());
    }
  }
  
  public static void init(IMemento memento) {
    filterList = new ArrayList<EvaluatorFilter>();
    EvaluatorFilter filter;
    JaninoEventEvaluator eval;
    IMemento[] mems = memento.getChildren(TAG_FILTER);
    for (int i = 0; i < mems.length; i++) {
      IMemento mem = mems[i];
      filter = new EvaluatorFilter();
      filter.setContext(context);
      filter.setName(NLS.bind(Messages.EventFilterName, i));
      filter.setOnMatch(FilterReply.valueOf(mem.getString(TAG_ONMATCH)));
      filter.setOnMismatch(FilterReply.valueOf(mem.getString(TAG_ONMISMATCH)));
      eval = new JaninoEventEvaluator();
      eval.setContext(context);
      eval.setName(NLS.bind(Messages.EventFilterEvalName, i));
      eval.setExpression(mem.getString(TAG_EXPRESSION));
      eval.start();
      filter.setEvaluator(eval);
      filter.start();
      filterList.add(filter);
    }
  }

  public static void moveFilterUp(EvaluatorFilter filter) {
    int index = filterList.indexOf(filter);
    if (index > 0) {
      filterList.remove(filter);
      filterList.add(index-1, filter);
    }
  }

  public static void moveFilterDown(EvaluatorFilter filter) {
    int index = filterList.indexOf(filter);
    if (index < filterList.size()-1) {
      filterList.remove(filter);
      filterList.add(index+1, filter);
    }
  }
  
}
