/**
 * Logback: the reliable, generic, fast and flexible logging framework.
 * 
 * Copyright (C) 1999-2007, QOS.ch
 * 
 * This library is free software, you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation.
 */
package ch.qos.logback.eclipse.reciever;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InvalidClassException;
import java.io.ObjectInputStream;
import java.net.Socket;

import org.eclipse.osgi.util.NLS;

import ch.qos.logback.classic.spi.LoggingEvent;
import ch.qos.logback.eclipse.Messages;
import ch.qos.logback.eclipse.PluginLogger;
import ch.qos.logback.eclipse.model.LoggingEventManager;

/**
 * Read {@link LoggingEvent} objects sent from a remote client using Sockets
 * (TCP).
 * 
 * @author Ceki G&uuml;lc&uuml;
 * @author S&eacute;bastien Pennec
 */
public class SocketNode implements Runnable {

  Socket socket;
  ObjectInputStream ois;
  boolean oisInError = false;

  public SocketNode(Socket socket) {
    this.socket = socket;
    try {
      ois = new ObjectInputStream(new BufferedInputStream(socket.getInputStream()));
    } catch (Exception e) {
      oisInError = true;
    }
  }

  public void run() {
    if (oisInError) {
      return;
    }
    
    LoggingEvent event;

    try {
      while (true) {
        // read an event from the wire
        event = (LoggingEvent) ois.readObject();
        // trick to keep the original thread name
        event.getThreadName();
        // add it to the manager's LoggingEvent list
        LoggingEventManager.getManager().addLoggingEvent(event);
      }
    } catch (ClassNotFoundException e) {
      printVersionError(e);
    } catch (InvalidClassException e) {
      printVersionError(e);
    } catch (java.io.EOFException e) {
      PluginLogger.logInfo(Messages.CaughtEOFException);
    } catch (java.net.SocketException e) {
    } catch (IOException e) {
      PluginLogger.logInfo(NLS.bind(Messages.CaughtIOException, e));
      PluginLogger.logInfo(Messages.ClosingConnection);
    } catch (Exception e) {
      PluginLogger.logError(NLS.bind(Messages.UnexpectedExceptionWithArg, Messages.ClosingConnection), e);
    }
    try {
      ois.close();
    } catch (Exception e) {
      PluginLogger.logError(Messages.CouldNotCloseConnection, e);
    }
  }
  
  private void printVersionError(Exception e) {
    PluginLogger.logError(NLS.bind(Messages.VersionError, Messages.RequiredLogBackVersion), e);
  }
  
}
